@@ -9,6 +9,7 @@ from .assign import PosixIdAssigner, AssignmentError, UpnAssigner
 from .config import Config, ConfigError
 from .paths import default_confpath
 from .state import DomainState
+from .staleacct import find_and_process_stale_accounts
 from .kerberos import setup_kerberos_environ, KerberosError
 from .version import __version__
 
@@ -95,12 +96,13 @@ def connect_ad(func):
             raise NotImplementedError("Only gssapi mode is supported")
 
         # TODO: Does this belong in ADManager?
-        setup_kerberos_environ(
-                username = lc.username,
-                realm = domain,
-                keytab = lc.keytab,
-                cache = lc.cache,
-            )
+        if lc.username:
+            setup_kerberos_environ(
+                    username = lc.username,
+                    realm = domain,
+                    keytab = lc.keytab,
+                    cache = lc.cache,
+                )
 
         ad = ADManager.connect(domain)
         return func(args, ad)
@@ -156,11 +158,20 @@ def cmd_exec(args, ad):
     sys.exit(rc)
 
 
+@connect_ad
+def cmd_findstale(args, ad):
+    sc = args.config.stale_accounts
+    if sc is None:
+        raise NotConfiguredError("stale_accounts")
+
+    find_and_process_stale_accounts(ad, args.config)
+
+
 @posix_assign
 def cmd_computer_assign(args, ad, pa):
     ia = args.config.id_assign
-    for container, scope in ia.only.iterate_containers():
-        pa.computer_assign(container=container, scope=scope)
+    for container, cfg in ia.only.iterate_containers():
+        pa.computer_assign(container=container, scope=cfg.scope)
 
 
 @connect_ad
@@ -171,10 +182,10 @@ def cmd_computer_list(args, ad):
 @posix_assign
 def cmd_group_assign(args, ad, pa):
     ia = args.config.id_assign
-    for container, scope in ia.only.iterate_containers():
+    for container, cfg in ia.only.iterate_containers():
         pa.assign_group_gidNumbers(
                 container = container,
-                scope = scope,
+                scope = cfg.scope,
                 include_computer_groups = ia.computers,
             )
 
@@ -187,8 +198,8 @@ def cmd_group_list(args, ad):
 @posix_assign
 def cmd_user_assign(args, ad, pa):
     ia = args.config.id_assign
-    for container, scope in ia.only.iterate_containers():
-        pa.user_assign(container=container, scope=scope)
+    for container, cfg in ia.only.iterate_containers():
+        pa.user_assign(container=container, scope=cfg.scope)
 
 
 @connect_ad
@@ -238,12 +249,12 @@ def cmd_user_mkdirs(args, ad):
                 acl = ud.acl,
             )
 
-        for container, scope in ud.only.iterate_containers():
+        for container, cfg in ud.only.iterate_containers():
             creator.create_userdirs(
                 basepath = ud.basepath,
                 tsd = tsd,
                 container = container,
-                scope = scope,
+                scope = cfg.scope,
                 subdirs = [
                     (
                         sd.name,
@@ -378,6 +389,11 @@ def parse_args():
     p.add_argument('args', nargs=argparse.REMAINDER,
             help="Command/args to exectue")
 
+    # findstale
+    p = sub.add_parser('findstale',
+            help="Find stale accounts and report/disable per config")
+    p.set_defaults(func=cmd_findstale)
+
 
     ######
     # computer
@@ -458,7 +474,7 @@ def parse_args():
             help="Check for expiring/expired passwords")
     p.set_defaults(func=cmd_user_checkexpire)
 
-    # user fixupn
+    # user setupns
     p = sp_user.add_parser('setupns',
             help="Set userPrincipalName attributes")
     p.set_defaults(func=cmd_user_setupns)
