@@ -1,5 +1,7 @@
 import yaml
+from datetime import timedelta
 from pathlib import Path
+
 from .util import join_nonempty
 
 ################################################################################
@@ -23,7 +25,9 @@ class ClassdataStrMixin:
                 yield '{}{}:'.format(' '*indent, k)
                 yield from get_child(indent+2)
             else:
-                yield '{}{}: {!r}'.format(' '*indent, k, v)
+                if isinstance(v, str):
+                    v = repr(v)
+                yield '{}{}: {}'.format(' '*indent, k, v)
 
 
 NODEFAULT = object()
@@ -39,24 +43,37 @@ class CfgBase(ClassdataStrMixin):
     def subkeyname(self, name):
         return join_nonempty('.', self.keyname, name)
 
-    def cfg_get(self, key, default=NODEFAULT, cls=None):
+    def cfg_get(self, key, defaultdata=NODEFAULT, default=NODEFAULT, cls=None):
         """Get an entry from the config data
 
         Parameters:
-        key     name of the entry to get
-        cls     optional class
+        key         name of the entry to get
+        default     default value to return directly if key is not set
+        cls         optional class or callable to invoke with data and return result
+        defaultdata default data to pass to cls
+
+        default and defaultdata are mutually exclusive.
+        defaultdata is only valid if cls is given.
         """
         try:
             value = self._data[key]
         except KeyError:
-            if default is NODEFAULT:
+            if defaultdata is NODEFAULT and default is NODEFAULT:
                 raise ConfigError("Missing key: {!r}".format(self.subkeyname(key)))
-            value = default
+
+            if default is not NODEFAULT:
+                assert defaultdata is NODEFAULT
+                return default
+
+            # Makes no sense to pass defaultdata but not cls
+            assert cls is not None
+            value = defaultdata
 
         if cls and value is not None:
-            # Ensure data is a dict unless the cls desires otherwise
-            if not isinstance(value, getattr(cls, 'data_type', dict)):
-                self.invalid_type(key, value)
+            # Ensure unprocessed input data is of the expected type
+            exp_type = cls.data_type
+            if not isinstance(value, exp_type):
+                self.invalid_type(key, value, exp_type)
             value = cls(data=value, name=key, parentnode=self)
 
         return value
@@ -84,8 +101,17 @@ class CfgBase(ClassdataStrMixin):
         """Raise a ConfigError for the given sub-key"""
         raise ConfigError("{}: {}".format(self.subkeyname(key), message))
 
-    def invalid_type(self, key, value):
-        self.raise_for_key(key, "Invalid type: {}".format(type(value).__name__))
+    def invalid_type(self, key, value, exp=None):
+        msg = "Invalid type: {}".format(type(value).__name__)
+        if exp:
+            if isinstance(exp, tuple):
+                expstr = '{' + ', '.join(t.__name__ for t in exp) + '}'
+            elif isinstance(exp, type):
+                expstr = exp.__name__
+            else:
+                assert False
+            msg += "  Expected: {}".format(expstr)
+        self.raise_for_key(key, msg)
 
 
 class CfgRoot(CfgBase):
@@ -103,6 +129,8 @@ class CfgRoot(CfgBase):
 
 
 class CfgNode(CfgBase):
+    data_type = dict
+
     def __init__(self, data, name, parentnode):
         assert name != None
         self._name = name
@@ -139,18 +167,47 @@ class CfgList(CfgNode, list):
                 yield '{}{}: {!r}'.format(' '*indent, k, v)
 
 
+class DurationConfig:
+    data_type = str
+
+    def __init__(self, data, name, parentnode):
+        assert isinstance(data, str)
+        self.__orig_str = data
+
+        try:
+            self.dt = self.parse(data)
+        except ValueError as e:
+            parentnode.raise_for_key(name, str(e))
+
+    @staticmethod
+    def parse(s):
+        parts = s.split()
+        if len(parts) != 2:
+            raise ValueError("Invalid duration (too many words): {}".format(parts))
+
+        n = int(parts[0])
+        units = parts[1].lower().rstrip('s')
+
+        mul = {
+            'second':   1,
+            'minute':   60,
+            'hour':     60 * 60,
+            'day':      60 * 60 * 24,
+            'week':     60 * 60 * 24 * 7,
+            'year':     60 * 60 * 24 * 365,
+        }.get(units)
+        if mul is None:
+            raise ValueError("Invalid units: {}".format(units))
+
+        return timedelta(seconds=n*mul)
+
+    def __str__(self):
+        return self.__orig_str
+
 
 ################################################################################
 # adman config
 
-class HasScope:
-    default_scope = 'subtree'
-
-    def check_scope(self):
-        if not self.scope in ('subtree', 'one'):
-            self.raise_for_key('scope', "Invalid scope: {}".format(self.scope))
-
-
 class LdapAuthConfigFactory(type):
     def __call__(cls, data, name, parentnode, **kwargs):
         if cls is not LdapAuthConfig:
@@ -177,36 +234,30 @@ class LdapAuthConfig(CfgNode, metaclass=LdapAuthConfigFactory):
         super().__init__(data=data, name=name, parentnode=parentnode)
 
 
+def all_or_none(*args):
+    nset = sum(bool(x) for x in args)
+    return nset in (0, len(args))
+
 class GssapiLdapAuthConfig(LdapAuthConfig):
     def _populate(self):
-        self.username = self.cfg_get('krb_username')
-        self.keytab   = self.cfg_get_path('krb_keytab')
-        self.cache    = self.cfg_get_path('krb_cache')
+        self.username = self.cfg_get('krb_username', default=None)
+        self.keytab   = self.cfg_get_path('krb_keytab', default=None)
+        self.cache    = self.cfg_get_path('krb_cache', default=None)
+
+        if not all_or_none(self.username, self.keytab, self.cache):
+            raise ConfigError("If any of the ldap_auth.krb_* args are set, all of them must be set")
 
 
-class UpnSuffixesConfig(CfgDict):
-    def _populate(self):
-        for container, d in self._data.items():
-            self[container] = UpnSuffixConfig(data=d, name=container, parentnode=self)
 
 
-class UpnSuffixConfig(CfgNode, HasScope):
-    def __init__(self, data, name, parentnode):
-        self.scope = self.default_scope
-        if isinstance(data, str):
-            self.suffix = data
-        elif isinstance(data, dict):
-            super().__init__(data=data, name=name, parentnode=parentnode)
-            self.suffix = self.cfg_get('suffix')
-            self.scope = self.cfg_get('scope', default=self.scope)
-        else:
-            parentnode.invalid_type(name, data)
 
-        self.check_scope()
 
 
 class PasswordExpConfig(CfgNode):
     def _populate(self):
+        if not self._parentnode.smtp:
+            self.raise_for_key('', "smtp not configured")
+
         # days
         days = self.cfg_get('days')
         if isinstance(days, int):
@@ -214,7 +265,7 @@ class PasswordExpConfig(CfgNode):
         elif isinstance(days, list):
             self.days = days
         else:
-            self.invalid_type('days', days)
+            self.invalid_type('days', days, (int, list))
         self.days.sort(reverse=True)
 
         # template
@@ -251,33 +302,70 @@ def RangeConfig(data, name, parentnode):
     o = CfgNode(data=data, name=name, parentnode=parentnode)
     return range(o.cfg_get('min'), o.cfg_get('max'))
 
+RangeConfig.data_type = dict      # XXX TODO: See isinstance check in CfgBase.cfg_get()
+
+
+class Container(CfgNode):
+    default_scope = 'subtree'
+
+    def __init__(self, data, name, parentnode):
+        if data is None:
+            data = {}
+        elif isinstance(data, dict):
+            pass
+        else:
+            parentnode.invalid_type(name, data)
+
+        super().__init__(data=data, name=name, parentnode=parentnode)
+
+        self.scope = self.cfg_get('scope', default=self.default_scope)
+        if not self.scope in ('subtree', 'one'):
+            self.raise_for_key('scope', "Invalid scope: {}".format(self.scope))
+
 
 class ContainerConfigDict(CfgDict):
+    value_class = Container
+
+    # all cfg_get() must use defaults if you use this!
+    dummy_class = Container
+
     def _populate(self):
+        assert issubclass(self.value_class, Container)
         for container, d in self._data.items():
-            self[container] = Container(data=d, name=container, parentnode=self)
+            self[container] = self.value_class(data=d, name=container, parentnode=self)
 
     def iterate_containers(self):
         if len(self) == 0:
             # Not configured => search everything
-            yield (None, 'subtree')
+            assert issubclass(self.dummy_class, Container)
+            dummy = self.dummy_class({}, 'dummy', self)
+            yield (None, dummy)
         else:
             for container, d in self.items():
-                yield (container, d.scope)
+                yield (container, d)
+
+    def _get_dict_for_str(self):
+        return dict(self.iterate_containers())
 
 
-class Container(CfgNode, HasScope):
+class UpnSuffixConfig(Container):
     def __init__(self, data, name, parentnode):
-        self.scope = self.default_scope
-        if data is None:
-            pass
+        if isinstance(data, str):
+            data = dict(suffix=data)
         elif isinstance(data, dict):
-            super().__init__(data=data, name=name, parentnode=parentnode)
-            self.scope = self.cfg_get('scope', default=self.scope)
+            pass
         else:
             parentnode.invalid_type(name, data)
 
-        self.check_scope()
+        super().__init__(data=data, name=name, parentnode=parentnode)
+
+    def _populate(self):
+        super()._populate()
+        self.suffix = self.cfg_get('suffix')
+
+
+class UpnSuffixesConfig(ContainerConfigDict):
+    value_class = UpnSuffixConfig
 
 
 class IdAssignConfig(CfgNode):
@@ -285,17 +373,17 @@ class IdAssignConfig(CfgNode):
         self.uid_range = self.cfg_get('uid_range', cls=RangeConfig)
         self.gid_range = self.cfg_get('gid_range', cls=RangeConfig)
         self.computers = self.cfg_get('computers', default=True)
-        self.only = self.cfg_get('only', default={}, cls=ContainerConfigDict)
+        self.only = self.cfg_get('only', defaultdata={}, cls=ContainerConfigDict)
 
 
 class UserdirConfig(CfgNode):
     def _populate(self):
         self.basepath = self.cfg_get('basepath')
-        self.only = self.cfg_get('only', default={}, cls=ContainerConfigDict)
+        self.only = self.cfg_get('only', defaultdata={}, cls=ContainerConfigDict)
         self.owner = self.cfg_get('owner', default=None)
         self.group = self.cfg_get('group', default=None)
         self.acl = self.cfg_get('acl', default=[])
-        self.subdirs = self.cfg_get('subdirs', default=[], cls=UsersubdirConfig)
+        self.subdirs = self.cfg_get('subdirs', defaultdata=[], cls=UsersubdirConfig)
 
 
 class UserdirsConfig(CfgList):
@@ -314,6 +402,35 @@ class UsersubdirConfig(CfgList):
             self.append(UserSubdirConfig(data=d, name=str(i), parentnode=self))
 
 
+class StaleAcctsConfig(CfgNode):
+    def _populate(self):
+        self.email_to = self.cfg_get('email_to', default=None)
+        if self.email_to and not self._parentnode.smtp:
+            self.raise_for_key('email_to', "smtp not configured")
+
+        # Global settings/defaults
+        self.older_than = self.cfg_get('older_than', cls=DurationConfig)
+        self.disable = self.cfg_get('disable', default=False)
+
+        def cont(key):
+            return self.cfg_get(key, defaultdata={}, cls=StaleAcctsContainerConfigDict)
+        self.users = cont('users')
+        self.computers = cont('computers')
+
+
+class StaleAcctsContainer(Container):
+    def _populate(self):
+        g = self._parentnode._parentnode
+
+        self.older_than = self.cfg_get('older_than', cls=DurationConfig, default=g.older_than)
+        self.disable = self.cfg_get('disable', default=g.disable)
+
+
+class StaleAcctsContainerConfigDict(ContainerConfigDict):
+    value_class = StaleAcctsContainer
+    dummy_class = StaleAcctsContainer
+
+
 class Config(CfgRoot):
     def __init__(self, data, path):
         cfgpath = Path(path).resolve(strict=True)
@@ -328,17 +445,16 @@ class Config(CfgRoot):
 
         self.ldap_auth = self.cfg_get('ldap_auth', cls=LdapAuthConfig)
 
-        self.userdirs = self.cfg_get('userdirs', default=[], cls=UserdirsConfig)
+        self.userdirs = self.cfg_get('userdirs', defaultdata=[], cls=UserdirsConfig)
 
-        self.upn_suffixes = self.cfg_get('upn_suffixes', default={}, cls=UpnSuffixesConfig)
-
-        self.pwexp = self.cfg_get('password_expiry_notification',
-                default=None, cls=PasswordExpConfig)
+        self.upn_suffixes = self.cfg_get('upn_suffixes', defaultdata={}, cls=UpnSuffixesConfig)
         
         self.smtp = self.cfg_get('smtp', default=None, cls=SmtpConfig)
 
-        if self.pwexp and not self.smtp:
-            raise ConfigError("smtp required if password_expiry_notification is used")
+        self.pwexp = self.cfg_get('password_expiry_notification',
+                default=None, cls=PasswordExpConfig)
+
+        self.stale_accounts = self.cfg_get('stale_accounts', default=None, cls=StaleAcctsConfig)
 
 
     @classmethod
@@ -356,11 +472,14 @@ if __name__ == '__main__':
     import argparse
     ap = argparse.ArgumentParser()
     ap.add_argument('config')
+    ap.add_argument('--debug', action='store_true')
     args = ap.parse_args()
 
     try:
         cfg = Config.load(args.config)
     except ConfigError as e:
+        if args.debug:
+            raise
         raise SystemExit("Config error: {}".format(e))
 
     print(cfg)
