@@ -4,13 +4,11 @@
 # https://github.com/sttts/ldap-notify (eDirectory, not Active Directory)
 
 from datetime import datetime, timedelta, timezone
-from email.message import EmailMessage
 import json
 import logging
-import smtplib
 from string import Template
 
-from .email import connect_smtp
+from .email import try_send_email
 from .ldapfilter import Filter, BitAndFilter, UserAccountControl
 from .util import single, utcnow
 from .paths import default_datadir
@@ -58,20 +56,12 @@ def _do_notify(now, user, config):
         # TODO: Remind in how many days?
     )
 
-    msg = EmailMessage()
-    msg['Subject'] = 'Password expiring'
-    msg['From'] = config.smtp.email_from
-    msg['To'] = user.mail
-    msg.set_content(body)
-
-    # Send it!
-    try:
-        with connect_smtp(config) as s:
-            s.send_message(msg)
-    except smtplib.SMTPException as e:
-        logger.exception(e)
-    except OSError as e:    # TimeoutError, ConnectionRefusedError
-        logger.exception(e)
+    try_send_email(
+            config = config,
+            mailto = user.mail,
+            subject = 'Password expiring',
+            body = body,
+        )
 
 
 def _should_notify(now, expiry, config, last_notify):
